/*
 * agent_sim.java
 * Copyright (c) 1996 Advent Network Management, Inc. All Rights Reserved.
 * Please read the associated COPYRIGHTS file for more details.
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation without fee is hereby granted provided 
 * that this copyright notice appears in all copies. 
 *
 * ADVENT NETWORK MANAGEMENT, INC. MAKES NO REPRESENTATIONS OR WARRANTIES 
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  ADVENT NETWORK 
 * MANAGEMENT, INC. SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY 
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE 
 * OR ITS DERIVATIVES.
 */

import java.lang.*;
import java.util.*;
import java.net.*;
import Snmp.*;
import ParseOptions;

/** This simple agent simulator allows you to feed it a MIB, or two,
and have it respond to get, getnext and set requests.  

The values returned are based on the DEFVAL clause.  Do not use quotes 
for non-strings.  For strings, one problem is thatquotes show up in the value.

Aside from ensuring an instance value, no checking of instances is done.  
The getnext returns an instance of 0 for everything.

It is far from a complete agent, so don't expect too much.  This is an 
exercise to help with building an agent */

public class agent_sim implements SnmpClient {
  SnmpAPI api;
  int errstat = 0;

  public static void main(String args[]) {
    agent_sim sim = new agent_sim();
    sim.init(args);
  } // end of main

  void init(String args[]) {
			// Take care of getting options
    String usage = "agent_sim [-d] [-c community] [-m MIB_file] [-m2 Second_MIB_file] [-p port]";
    String options[] = { "-d", "-c", "-m", "-m2", "-p"};
    String values[] = { "None", null,null, null, null};
      
    ParseOptions opt = new ParseOptions(args,options,values, usage);

			// Start SNMP API
    api = new SnmpAPI();
    api.start();
    api.client = this;  // for callbacks
    if (values[0].equals("Set")) api.DEBUG = true;

			// Next deal with MIBs
    MibModule m1, m2;
    if ( (values[2] !=null)||(values[3] !=null) ) try { // Open the MIB module 
	System.out.println("Loading MIB files: " + values[2] +" "+ values[3]);
	if (values[2] != null) m1 = new MibModule(values[2], api,api.DEBUG);
	if (values[3] != null) m2 = new MibModule(values[3], api,api.DEBUG);
	System.out.println("Done.");

    } catch(Exception e)  { System.err.println(e); }


    if (opt.remArgs.length>0) opt.usage_error();

		// Open session and set local port
    SnmpSession session = new SnmpSession(api);

    if (values[1] != null) session.community = values[1];
    else session.community = "public";

    try {  // use specified or standard SNMP port
      if (values[4] != null) session.local_port = Integer.parseInt(values[4]);
      else session.local_port = api.SNMP_PORT;
    }
    catch (NumberFormatException ex) { System.err.println("Invalid Integer Arg"); } 

	// Open the session
    try { session.open(); }
    catch (SnmpException e) { 
      System.err.println(e); 
      System.exit(1);
    }

  } // end of init

/** The callback for incoming PDUs */
  public boolean callback(SnmpSession session, SnmpPDU pdu, int reqid) {
    int req = pdu.command;
    pdu.command = SnmpAPI.GET_RSP_MSG;
    int i, numVarBinds = pdu.variables.size();

    try { // pick up any send exceptions 

      if (numVarBinds == 0) {session.send(pdu);return true;} // no varbinds

      for (i=0;i<numVarBinds;i++) {
	SnmpVarBind varbind = (SnmpVarBind) pdu.variables.elementAt(i);
	try { varbind.variable = getValue(varbind,req); }
	catch (MibException me) {System.err.println(me);break;} // errors
      }
      if (i < numVarBinds) {pdu.errindex = i+1;pdu.errstat = errstat;}
      session.send(pdu);
    } catch (SnmpException e) { System.err.println(e); }

    return true; // no more processing required in SnmpSession
  }

/** returns the variable requested */
  SnmpVar getValue(SnmpVarBind varbind, int req) throws MibException {
    MibNode node = varbind.oid.node;

    if (req == SnmpAPI.GET_REQ_MSG) {  // get request
	// reject if no MIB node, not leaf node, no instance specified
      if ((node == null) || (node.syntax == null) ||
	  (node.getOID().length >= ((long[])varbind.oid.toValue()).length) ) {
	if (node.parent != null) if (node.parent.indexNames != null) {
	  
	}
	errstat = 2;
	throw new MibException("No MIB Node, not a leaf node, or no instance.");
      } else if (node.defval == null) return new SnmpNull();
	try { return(node.syntax.createVariable(node.defval)); }
	catch (SnmpException e) {
		errstat = 3;
		throw new MibException("Invalid DEFVAL: "+node.defval);
	     }

    } else if (req == SnmpAPI.GETNEXT_REQ_MSG) {
	// if no node found found for oid, use the root in first MIB
      if (node == null) node = ((MibModule)api.modules.firstElement()).root;

      node = getNextNode(node, varbind);
      if (node == null) 
	{errstat = 2; throw new MibException("No next node in this MIB");}
      if (node.defval == null) return new SnmpNull();
      try { return(node.syntax.createVariable(node.defval)); }
      catch (SnmpException e) {
		errstat = 3;
		throw new MibException("Invalid DEFVAL: "+node.defval);
      }

    } else if (req == SnmpAPI.SET_REQ_MSG) { // set request
	// reject if no MIB node, not leaf node, no instance specified
      if ((node == null) || (node.syntax == null) ||
	  (node.getOID().length >= ((long[])varbind.oid.toValue()).length ) ) {
	errstat = 2;
	throw new MibException("No MIB Node, not a leaf node, or no instance");
      } else {
	if (varbind.variable.Type != node.syntax.type) {
		errstat = 3;
		throw new MibException("Invalid Syntax: "+varbind.variable);
	} else { 
	   if (varbind.variable.Type == SnmpAPI.OBJID) 
		node.defval = varbind.variable.toString();
	   else if (varbind.variable instanceof SnmpIpAddress) {
		node.defval = varbind.variable.toString();
		int i = node.defval.indexOf("/");
		if (i!=-1) node.defval = node.defval.substring(0,i);
	   } else node.defval = varbind.variable.toValue().toString();
System.err.println("Addr: "+node.defval);
	   try { return(node.syntax.createVariable(node.defval)); }
	   catch (SnmpException e) {
		errstat = 3;
		throw new MibException("Invalid DEFVAL: "+node.defval);
	    }

	}

      } // end set request
    }

    return null; 
  }

/** get the next node required for getnext */
  MibNode getNextNode(MibNode node, SnmpVarBind varbind) {

	if ((node.getOID().length == ((long[])varbind.oid.toValue()).length)
	        && (node.syntax == null) )  {
		// means the oid is not the required node w/o the instance
		if  ((node = node.getNextLeafNode()) == null) return null;
	}

	if (node.getOID().length < ((long[])varbind.oid.toValue()).length)
	// means the oid is not the required node w/o the instance
		if  ((node = node.getNextLeafNode()) == null) return null;

    varbind.oid = new SnmpOID(node.getNumberedOIDString() + ".0");
    return node;
  }

 /** We need to implement the other methods in the SnmpClient interface */
  public void debugPrint(String s) {System.err.println(s);}
  public boolean authenticate(SnmpPDU pdu, String community) {
	if (pdu.community.equals(community)) return true;
	return false;  
  }

}

